# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'maskWidgetUI.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(363, 366)
        self.horizontalLayout = QtWidgets.QHBoxLayout(Form)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.toolsFrame = QtWidgets.QFrame(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.toolsFrame.sizePolicy().hasHeightForWidth())
        self.toolsFrame.setSizePolicy(sizePolicy)
        self.toolsFrame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.toolsFrame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.toolsFrame.setObjectName("toolsFrame")
        self.gridLayout = QtWidgets.QGridLayout(self.toolsFrame)
        self.gridLayout.setObjectName("gridLayout")
        self.lineMaskButton = QtWidgets.QPushButton(self.toolsFrame)
        self.lineMaskButton.setObjectName("lineMaskButton")
        self.gridLayout.addWidget(self.lineMaskButton, 3, 0, 1, 1)
        self.frame = QtWidgets.QFrame(self.toolsFrame)
        self.frame.setMinimumSize(QtCore.QSize(150, 150))
        self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.frame)
        self.gridLayout_2.setContentsMargins(1, 1, 1, 1)
        self.gridLayout_2.setSpacing(1)
        self.gridLayout_2.setObjectName("gridLayout_2")
        spacerItem = QtWidgets.QSpacerItem(88, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem, 2, 2, 1, 1)
        self.xMinBox = QtWidgets.QSpinBox(self.frame)
        self.xMinBox.setObjectName("xMinBox")
        self.gridLayout_2.addWidget(self.xMinBox, 3, 2, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(88, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem1, 3, 1, 1, 1)
        self.yMinBox = QtWidgets.QSpinBox(self.frame)
        self.yMinBox.setObjectName("yMinBox")
        self.gridLayout_2.addWidget(self.yMinBox, 2, 1, 1, 1)
        self.xMaxBox = QtWidgets.QSpinBox(self.frame)
        self.xMaxBox.setObjectName("xMaxBox")
        self.gridLayout_2.addWidget(self.xMaxBox, 1, 2, 1, 1)
        spacerItem2 = QtWidgets.QSpacerItem(88, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem2, 3, 3, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(88, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem3, 1, 1, 1, 1)
        spacerItem4 = QtWidgets.QSpacerItem(88, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem4, 1, 3, 1, 1)
        self.yMaxBox = QtWidgets.QSpinBox(self.frame)
        self.yMaxBox.setObjectName("yMaxBox")
        self.gridLayout_2.addWidget(self.yMaxBox, 2, 3, 1, 1)
        self.label = QtWidgets.QLabel(self.frame)
        self.label.setObjectName("label")
        self.gridLayout_2.addWidget(self.label, 4, 0, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.frame)
        self.label_2.setObjectName("label_2")
        self.gridLayout_2.addWidget(self.label_2, 0, 4, 1, 1)
        self.yMaxSlider = QtWidgets.QSlider(self.frame)
        self.yMaxSlider.setOrientation(QtCore.Qt.Vertical)
        self.yMaxSlider.setObjectName("yMaxSlider")
        self.gridLayout_2.addWidget(self.yMaxSlider, 1, 4, 3, 1)
        self.yMinSlider = QtWidgets.QSlider(self.frame)
        self.yMinSlider.setOrientation(QtCore.Qt.Vertical)
        self.yMinSlider.setObjectName("yMinSlider")
        self.gridLayout_2.addWidget(self.yMinSlider, 1, 0, 3, 1)
        self.xMaxSlider = QtWidgets.QSlider(self.frame)
        self.xMaxSlider.setOrientation(QtCore.Qt.Horizontal)
        self.xMaxSlider.setObjectName("xMaxSlider")
        self.gridLayout_2.addWidget(self.xMaxSlider, 0, 1, 1, 3)
        self.xMinSlider = QtWidgets.QSlider(self.frame)
        self.xMinSlider.setOrientation(QtCore.Qt.Horizontal)
        self.xMinSlider.setObjectName("xMinSlider")
        self.gridLayout_2.addWidget(self.xMinSlider, 4, 1, 1, 3)
        self.gridLayout.addWidget(self.frame, 4, 0, 1, 2)
        self.ellipseMaskButton = QtWidgets.QPushButton(self.toolsFrame)
        self.ellipseMaskButton.setObjectName("ellipseMaskButton")
        self.gridLayout.addWidget(self.ellipseMaskButton, 2, 1, 1, 1)
        self.addROI = QtWidgets.QRadioButton(self.toolsFrame)
        self.addROI.setChecked(True)
        self.addROI.setObjectName("addROI")
        self.gridLayout.addWidget(self.addROI, 1, 0, 1, 1)
        self.rectMaskButton = QtWidgets.QPushButton(self.toolsFrame)
        self.rectMaskButton.setObjectName("rectMaskButton")
        self.gridLayout.addWidget(self.rectMaskButton, 2, 0, 1, 1)
        self.subtractROI = QtWidgets.QRadioButton(self.toolsFrame)
        self.subtractROI.setObjectName("subtractROI")
        self.gridLayout.addWidget(self.subtractROI, 1, 1, 1, 1)
        self.polyMaskButton = QtWidgets.QPushButton(self.toolsFrame)
        self.polyMaskButton.setObjectName("polyMaskButton")
        self.gridLayout.addWidget(self.polyMaskButton, 3, 1, 1, 1)
        self.archList = QtWidgets.QComboBox(self.toolsFrame)
        self.archList.setObjectName("archList")
        self.gridLayout.addWidget(self.archList, 0, 0, 1, 1)
        self.clearButton = QtWidgets.QPushButton(self.toolsFrame)
        self.clearButton.setObjectName("clearButton")
        self.gridLayout.addWidget(self.clearButton, 0, 1, 1, 1)
        self.setCurrent = QtWidgets.QPushButton(self.toolsFrame)
        self.setCurrent.setObjectName("setCurrent")
        self.gridLayout.addWidget(self.setCurrent, 5, 0, 1, 1)
        self.setGlobal = QtWidgets.QPushButton(self.toolsFrame)
        self.setGlobal.setObjectName("setGlobal")
        self.gridLayout.addWidget(self.setGlobal, 5, 1, 1, 1)
        self.horizontalLayout.addWidget(self.toolsFrame)

        self.retranslateUi(Form)
        self.xMaxBox.valueChanged['int'].connect(self.xMaxSlider.setValue)
        self.xMaxSlider.sliderMoved['int'].connect(self.xMaxBox.setValue)
        self.yMaxBox.valueChanged['int'].connect(self.yMaxSlider.setValue)
        self.yMaxSlider.sliderMoved['int'].connect(self.yMaxBox.setValue)
        self.xMinBox.valueChanged['int'].connect(self.xMinSlider.setValue)
        self.xMinSlider.sliderMoved['int'].connect(self.xMinBox.setValue)
        self.yMinSlider.sliderMoved['int'].connect(self.yMinBox.setValue)
        self.yMinBox.valueChanged['int'].connect(self.yMinSlider.setValue)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.lineMaskButton.setText(_translate("Form", "Line"))
        self.label.setText(_translate("Form", "Min"))
        self.label_2.setText(_translate("Form", "Max"))
        self.ellipseMaskButton.setText(_translate("Form", "Ellipse"))
        self.addROI.setText(_translate("Form", "Add"))
        self.rectMaskButton.setText(_translate("Form", "Rect"))
        self.subtractROI.setText(_translate("Form", "Subtract"))
        self.polyMaskButton.setText(_translate("Form", "Poly"))
        self.clearButton.setText(_translate("Form", "Clear"))
        self.setCurrent.setText(_translate("Form", "Set Current"))
        self.setGlobal.setText(_translate("Form", "Set Global"))
