# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'c:\Users\walroth\Documents\repos\xdart\xdart\experiments\ttheta_scan\gui\integratorUI.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(218, 264)
        self.verticalLayout = QtWidgets.QVBoxLayout(Form)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.frame1D = QtWidgets.QFrame(Form)
        self.frame1D.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame1D.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame1D.setObjectName("frame1D")
        self.layout1D = QtWidgets.QVBoxLayout(self.frame1D)
        self.layout1D.setContentsMargins(0, 0, 0, 0)
        self.layout1D.setSpacing(0)
        self.layout1D.setObjectName("layout1D")
        self.label = QtWidgets.QLabel(self.frame1D)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.label.setAlignment(QtCore.Qt.AlignCenter)
        self.label.setObjectName("label")
        self.layout1D.addWidget(self.label)
        self.frame_3 = QtWidgets.QFrame(self.frame1D)
        self.frame_3.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_3.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_3.setLineWidth(0)
        self.frame_3.setObjectName("frame_3")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.frame_3)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setSpacing(2)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.integrate1D = QtWidgets.QPushButton(self.frame_3)
        self.integrate1D.setObjectName("integrate1D")
        self.horizontalLayout.addWidget(self.integrate1D)
        self.advanced1D = QtWidgets.QPushButton(self.frame_3)
        self.advanced1D.setObjectName("advanced1D")
        self.horizontalLayout.addWidget(self.advanced1D)
        self.all1D = QtWidgets.QCheckBox(self.frame_3)
        self.all1D.setObjectName("all1D")
        self.horizontalLayout.addWidget(self.all1D)
        self.layout1D.addWidget(self.frame_3)
        self.verticalLayout.addWidget(self.frame1D)
        self.frame2D = QtWidgets.QFrame(Form)
        self.frame2D.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame2D.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame2D.setObjectName("frame2D")
        self.layout2D = QtWidgets.QVBoxLayout(self.frame2D)
        self.layout2D.setContentsMargins(0, 0, 0, 0)
        self.layout2D.setSpacing(0)
        self.layout2D.setObjectName("layout2D")
        self.label_2 = QtWidgets.QLabel(self.frame2D)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_2.setFont(font)
        self.label_2.setAlignment(QtCore.Qt.AlignCenter)
        self.label_2.setObjectName("label_2")
        self.layout2D.addWidget(self.label_2)
        self.frame_4 = QtWidgets.QFrame(self.frame2D)
        self.frame_4.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame_4.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_4.setObjectName("frame_4")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.frame_4)
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_2.setSpacing(2)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.integrate2D = QtWidgets.QPushButton(self.frame_4)
        self.integrate2D.setObjectName("integrate2D")
        self.horizontalLayout_2.addWidget(self.integrate2D)
        self.advanced2D = QtWidgets.QPushButton(self.frame_4)
        self.advanced2D.setObjectName("advanced2D")
        self.horizontalLayout_2.addWidget(self.advanced2D)
        self.all2D = QtWidgets.QCheckBox(self.frame_4)
        self.all2D.setObjectName("all2D")
        self.horizontalLayout_2.addWidget(self.all2D)
        self.layout2D.addWidget(self.frame_4)
        self.verticalLayout.addWidget(self.frame2D)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.label.setText(_translate("Form", "1D"))
        self.integrate1D.setText(_translate("Form", "Integrate"))
        self.advanced1D.setText(_translate("Form", "Advanced..."))
        self.all1D.setText(_translate("Form", "All"))
        self.label_2.setText(_translate("Form", "2D"))
        self.integrate2D.setText(_translate("Form", "Integrate"))
        self.advanced2D.setText(_translate("Form", "Advanced..."))
        self.all2D.setText(_translate("Form", "All"))

